try:
    bytes('A', 'latin1')
except TypeError:
    bytes = lambda a, b: a  # noqa
    chr = lambda a: a if type(a) == str else chr(a)  # noqa


def null_padded(string, length):
    """
    Force a string to a given length by right-padding it with zero-bytes,
    clipping the initial string if neccessary.
    """
    return bytes(string[:length] + ('\0' * (length - len(string))), 'latin1')



from PIL import Image
from elma.constants import LGR_DEFAULT_PALETTE
from elma.constants import LGR_END_OF_FILE
from elma.constants import LGR_FOOD_NAME
from elma.constants import LGR_NOT_IN_PICTURES_LST
from elma.constants import LGR_OBJECT_NAME
from elma.constants import LGR_PCX_PADDING
from elma.constants import LGR_PICTURES_LST_ID
from elma.utils import null_padded
import io
import re
import struct


class LGR_Image(object):
    """
    Represents one entry in an LGR file.

    All entries in the LGR file have a name and an image (in .pcx format). Most
    images have extra information stored in a pictures.lst file contained
    within the LGR file. However, some of the special images (bike images and
    qkiller, qexit, qframe, qcolors and qgrass) are not included in the
    pictures.lst file and so do not have this additional information.

    Attributes:
        name (string): The name of the item in the LGR file, excluding pcx.
            e.g. "barrel"
        img (Image): Image object pointing to the image to be used
            (PIL.Image.open(file))
            http://pillow.readthedocs.io/en/3.1.x/reference/Image.html
        image_type (int): The type of image. Should be one of:
            LGR_Image.PICTURE, LGR_Image.TEXTURE, LGR_Image.MASK. Only used if
            **is_in_pictures_lst()=True**
        default_distance (int): The default z-ordering distance of a picture or
            texture. Should be in the range 1-999. Only used **if
            is_in_pictures_lst()=True**
        default_clipping (int): The default clipping of a picture or texture.
            Should be one of: LGR_Image.CLIPPING_U, LGR_Image.CLIPPING_G,
            LGR_Image.CLIPPING_S. Only used if **is_in_pictures_lst()=True**
        transparency (int): The pixel corresponding to the color or item in a
        palette that should be transperant. Should be one of:
            LGR_Image.TRANSPARENCY_PAL_ZERO, LGR_Image.TRANSPARENCY_TOPLEFT,
            LGR_Image.TRANSPARENCY_TOPRIGHT, LGR_Image.TRANSPARENCY_BOTTOMLEFT,
            LGR_Image.TRANSPARENCY_BOTTOMRIGHT. Only used if
            **is_in_pictures_lst()=True**.  TRANSPARENCY_PAL_ZERO indicates the
            the palette index 0 is selected as the transparent color.
        padding (int[7]): Each LGR entry has 7 bytes of padding that are
            unused. This can in theory be used to store extra information.
    """

    CLIPPING_U = 0
    CLIPPING_G = 1
    CLIPPING_S = 2
    PICTURE = 100
    TEXTURE = 101
    MASK = 102
    TRANSPARENCY_PAL_ZERO = 11
    TRANSPARENCY_TOPLEFT = 12
    TRANSPARENCY_TOPRIGHT = 13
    TRANSPARENCY_BOTTOMLEFT = 14
    TRANSPARENCY_BOTTOMRIGHT = 15

    def is_in_pictures_lst(self):
        """
        Determines whether image with the given name appears in pictures.lst
        (i.e. has the extra variables image_type, default_distance,
        default_clipping, transparency)
        Returns True if the image appears in pictures.lst, meaning the extra
        variables may be used. See also is_special().
        """
        return self.name.lower() not in LGR_NOT_IN_PICTURES_LST

    def get_palette(self):
        """
        Returns the palette of the image in the format [r, g, b, ...], or None
        if the image has no palette.
        """
        return self.img.getpalette()

    def put_palette(self, palette):
        """
        Sets the image's palette. Does NOT convert image colors. Can only be
        used on images that are palette-based.
        """
        self.img.putpalette(palette)

    def convert_palette_image(self, palette_info=LGR_DEFAULT_PALETTE,
                              dither=False):
        """
        Converts the current image into an appropriate palette
        format, keeping the color as close to the original as possible.

        Arguments:
            palette_info (byte[768]): The palette to apply to the image. May be
                obtained from LGR_Image.get_palette(). The default is the
                default palette from default.lgr.
            dither (bool): Whether to dither the image during conversion
        """
        palette = Image.new('P', (1, 1))
        palette.putpalette(palette_info[:], 'RGB')
        self.img = self.img._new(
            self.img.convert(
                mode='RGB').im.convert('P', dither, palette.im))

    def is_valid_palette_image(self):
        """
        Returns True if image has a palette of 256 rgb values.
        """
        return (self.img.mode == 'P' and
                self.img.palette.mode == 'RGB' and
                len(self.get_palette()) == 768)

    def save_PCX(self, filename):
        """
        Writes the image as a .pcx file to `filename`.
        """
        self.img.save(filename, 'pcx')

    def is_qup_qdown(self):
        """
        Checks if the current object name qualifies as a qup_ or qdown_ object.
        """
        name_lower = self.name.lower()
        return name_lower[:4] == "qup_" or name_lower[:6] == "qdown_"

    def is_food(self):
        """
        Checks if the current object name is qfood*.
        """
        return self.name.lower() in LGR_FOOD_NAME

    def is_object(self):
        """
        Checks if the current name is a recognized object (qfood*, qexit,
        qkiller)
        """
        return (self.name.lower() in LGR_OBJECT_NAME)

    def is_special(self):
        """
        Checks if the current object is a special object (i.e. treated
        differently from normal images, so that image_type, default_distance,
        default_clipping, trasparency either don't exist or are ignored by
        elma.exe.
        """
        return (not self.is_in_pictures_lst() or
                self.is_food() or
                self.is_qup_qdown())

    def __init__(self, name, img=None, image_type=PICTURE,
                 default_distance=500, default_clipping=CLIPPING_S,
                 transparency=TRANSPARENCY_TOPLEFT, padding=LGR_PCX_PADDING):
        self.name = name
        self.img = img
        self.padding = padding
        self.image_type = image_type
        self.default_distance = default_distance
        self.default_clipping = default_clipping
        self.transparency = transparency

    def __repr__(self):
        if(self.is_in_pictures_lst()):
            return (('LGR_Image(name: %s, img: %s, image_type: %s, ' +
                     'default_distance: %s, default_clipping: %s, ' +
                     'transparency: %s, padding: %s)') %
                    (self.name, self.img, self.image_type,
                     self.default_distance, self.default_clipping,
                     self.transparency, self.padding))
        return (('LGR_Image(name: %s, img: %s, padding: %s)') %
                (self.name, self.img, self.padding))

    def __eq__(self, other_picture):
        if(self.is_in_pictures_lst()):
            return (self.name == other_picture.name and
                    self.img.mode == other_picture.img.mode and
                    self.img.size == other_picture.img.size and
                    self.get_palette() == other_picture.get_palette() and
                    self.img.tobytes() == other_picture.img.tobytes() and
                    self.image_type == other_picture.image_type and
                    self.default_distance == other_picture.default_distance and
                    self.default_clipping == other_picture.default_clipping and
                    self.transparency == other_picture.transparency and
                    self.padding == other_picture.padding)
        return (self.name == other_picture.name and
                self.img.mode == other_picture.img.mode and
                self.img.size == other_picture.img.size and
                self.get_palette() == other_picture.get_palette() and
                self.img.tobytes() == other_picture.img.tobytes() and
                self.padding == other_picture.padding)


class LGR(object):
    """
    Represent an LGR file

    Attributes:
        images (list): A list of LGR_Image objects contained in the lgr file
        palette: Palette to use in the LGR file. Should be an array 768 bytes
            long in the format [r,g,b,r,g,b,...]. Inputting -1 will use the
            default palette from default.lgr
    """

    def __init__(self, palette=None):
        self.images = []
        if not palette:
            palette = LGR_DEFAULT_PALETTE[:]
        self.palette = palette

    def find_LGR_Image(self, filename):
        """
        Searches for an LGR_Image with the name of "filename" and returns the
        index number in the list of LGR.images. Case-insensitive. Returns
        ValueError if not found.
        """
        filename = filename.lower()
        for i, image in enumerate(self.images):
            if image.name.lower() == filename:
                return i
        raise ValueError('\'%s\' not in LGR.images' % filename)

    def __repr__(self):
        return 'LGR(images: %s)' % self.images


def unpack_LGR(data):
    """
    Opens an LGR file when you pass raw data or a filename
    """

    def get_int32(loc):
        return struct.unpack('<I', data[loc:loc+4])[0]

    if(type(data).__name__ == "str"):
        with open(data, 'rb') as f:
            data = f.read()

    lgr = LGR()
    pictures = []  # temp list to retain order of files by x instead of l

    assert(data[0:5] == b'LGR12')
    x = get_int32(5)
    assert(get_int32(9) == LGR_PICTURES_LST_ID)
    l = get_int32(13)

    for i in range(l):
        pictures.append(LGR_Image(
            name=data[17+10*i:17+10*i+10].rstrip(b'\0').decode('latin1'),
            img=None,
            image_type=get_int32(17+10*l+4*i),
            default_distance=get_int32(17+14*l+4*i),
            default_clipping=get_int32(17+18*l+4*i),
            transparency=get_int32(17+22*l+4*i),
            padding=LGR_PCX_PADDING))

    sp = 17+l*26

    found_palette = False
    term = re.compile(b'.pcx\0', re.IGNORECASE)
    for i in range(x):
        pcx_name = data[sp:sp+13]
        pcx_name = pcx_name[:term.search(pcx_name).start()].decode('latin1')
        lst_pcx_match = False
        pcx_padding = [int.from_bytes(data[sp+12+1+k:sp+12+1+k+1],
                                      byteorder='little', signed=False)
                       for k in range(7)]
        pcx_len = get_int32(sp+20)
        pcx_img = Image.open(io.BytesIO(data[sp+24:sp+24+pcx_len]))
        if pcx_name.lower() == 'q1bike':
            lgr.palette = pcx_img.getpalette()
            found_palette = True
        for obj in pictures:
            if(obj.name.lower() == pcx_name.lower()):
                obj.img = pcx_img
                obj.padding = pcx_padding
                lgr.images.append(obj)
                lst_pcx_match = True
                break
        if not lst_pcx_match:
            lgr.images.append(LGR_Image(
                name=pcx_name,
                img=pcx_img,
                padding=pcx_padding))
        sp = sp+24+pcx_len
    if not found_palette and lgr.images:
        lgr.images[0].get_palette()

    assert(get_int32(sp) == LGR_END_OF_FILE)

    return lgr


def pack_LGR(lgr):
    """
    Converts LGR object into its binary representation to be saved as an lgr
    file.
    """

    def to_int32(val):
        return struct.pack('<I', val)

    l = 0
    l_name = []
    l_image_type = []
    l_default_distance = []
    l_default_clipping = []
    l_transparency = []
    x = []
    for obj in lgr.images:
        if(obj.is_in_pictures_lst()):
            l = l+1
            l_name.extend([null_padded(obj.name, 8), b'\0\0'])
            l_image_type.append(to_int32(obj.image_type))
            l_default_distance.append(to_int32(obj.default_distance))
            l_default_clipping.append(to_int32(obj.default_clipping))
            l_transparency.append(to_int32(obj.transparency))
        with io.BytesIO() as f:
            obj.save_PCX(f)
            x_len = f.tell()
            f.seek(0)
            x.extend([
                null_padded('%s.pcx' % obj.name, 13),
                bytes(obj.padding),
                to_int32(x_len),
                f.read()])

    return b"".join([
        b'LGR12',
        to_int32(len(lgr.images)),
        to_int32(LGR_PICTURES_LST_ID),
        to_int32(l),
        b''.join(l_name),
        b''.join(l_image_type),
        b''.join(l_default_distance),
        b''.join(l_default_clipping),
        b''.join(l_transparency),
        b''.join(x),
        to_int32(LGR_END_OF_FILE)])



from elma.lgr import LGR_Image
from elma.constants import LGR_MANDATORY_FILES
from elma.constants import LGR_LIMITED_SIZE_FILES
import io

LGR_PCX_MIN = 10    # unused as practically never important
LGR_PCX_MAX = 3500    # unused as practically never important
LGR_PCX_FILESIZE_MIN = 1    # unused as practically never important
LGR_PCX_FILESIZE_MAX = 10000000
LGR_PIC_MAX = 999
LGR_PIC_WIDTH_MAX = 6000
LGR_PIC_SIZE_MAX = 600000
LGR_TEX_MIN = 2
LGR_TEX_MAX = 99
LGR_MASK_MAX = 199
LGR_GRASS_MAX = 99
LGR_OBJ_WIDTH = 40
LGR_OBJ_HEIGHT = 40
LGR_OBJ_FRAMES_MAX = 1000
LGR_RECOMMEND_BIKE_WIDTH = 354
LGR_RECOMMEND_BIKE_HEIGHT = 298
LGR_WARNING_GRASS_HEIGHT_MIN = 41
LGR_WARNING_QCOLORS_WIDTH = 66
LGR_WARNING_QCOLORS_HEIGHT = 109

ERR_LGR_INVALID_PALETTE = 1
ERR_LGR_MISSING_MANDATORY_FILE = 2
ERR_DUPLICATE_NAME = 3
ERR_TOO_MANY_GRASS = 4
ERR_TOO_MANY_TEXTURES = 5
ERR_NOT_ENOUGH_TEXTURES = 6
ERR_TOO_MANY_PICTURES = 7
ERR_TOO_MANY_MASKS = 8
WARN_UNUSED_QFOOD = 501

ERR_FILE_TOO_LARGE = 1001
ERR_NAME_TOO_LONG = 1002
ERR_NAME_MISSING = 1003
ERR_PADDING_INVALID = 1004
ERR_IMAGE_TYPE_INVALID = 1005
ERR_DISTANCE_INVALID = 1006
ERR_TRANSPARENCY_INVALID = 1007
ERR_PIC_TOO_WIDE = 1008
ERR_PIC_TOO_MANY_PIXELS = 1009
ERR_OBJ_WIDTH_INVALID = 1010
ERR_OBJ_TOO_WIDE = 1011
ERR_SMALL_IMAGE_TOO_LARGE = 1012
ERR_IMAGE_INVALID_PALETTE = 1013
ERR_IMG_MISSING = 1014
ERR_CLIPPING_INVALID = 1015

WARN_OBJ_HEIGHT_INVALID = 5001
WARN_GRASS_HEIGHT_TOO_SMALL = 5002
WARN_PALETTE_MISMATCH = 5003
WARN_QCOLORS_WRONG_SIZE = 5004
WARN_QBIKE_TOO_SMALL = 5005


def check_LGR_error(lgro, palette=None):
    """
    Returns a list of errors or warnings for an LGR object or LGR_Image object.
    If you pass an LGR_Image object, you can also put a palette that the object
    should have
    """
    message = []
    if(type(lgro).__name__ == "LGR"):
        use_palette = None
        if(len(lgro.palette) == 768 and
           max(lgro.palette) <= 255 and
           min(lgro.palette) >= 0):
            use_palette = lgro.palette
        else:
            message.append([
                ERR_LGR_INVALID_PALETTE,
                None,
                "The LGR file has an invalid palette! Please set a palette "
                "using get_palette() on one of the images or using "
                "LGR_Image.default_palette()"])
        n_pic = 0
        n_tex = 0
        n_mask = 0
        n_grass = 0
        apples = [False] * 9
        for item in LGR_MANDATORY_FILES:
            n_count = 0
            for obj in lgro.images:
                if(item == obj.name.lower()):
                    n_count = 1
                    break
            if(n_count == 0):
                message.append([
                    ERR_LGR_MISSING_MANDATORY_FILE,
                    item,
                    "The LGR file is missing a mandatory file: %s" % item])
        len_lgr = len(lgro.images)
        for i in range(len_lgr):
            namelower = lgro.images[i].name.lower()
            if(lgro.images[i].is_in_pictures_lst()):
                if(lgro.images[i].is_qup_qdown()):
                    n_grass += 1
                elif(lgro.images[i].is_food()):
                    apples[int(namelower[5:6])-1] = True
                elif(lgro.images[i].image_type == LGR_Image.PICTURE):
                    n_pic += 1
                elif(lgro.images[i].image_type == LGR_Image.TEXTURE):
                    n_tex += 1
                elif(lgro.images[i].image_type == LGR_Image.MASK):
                    n_mask += 1
            else:
                if(namelower == "qgrass"):
                    n_tex += 1

            for j in range(i+1, len_lgr, 1):
                if(namelower == lgro.images[j].name.lower()):
                        message.append([
                            ERR_DUPLICATE_NAME,
                            lgro.images[j],
                            "The LGR file has a duplicate of the "
                            "following filename: %s" % lgro.images[j].name])
            message_temp = check_LGR_error(lgro.images[i], use_palette)
            if(message_temp):
                message.extend(message_temp)
        if(n_grass > LGR_GRASS_MAX):
            message.append([
                ERR_TOO_MANY_GRASS,
                n_grass,
                "The LGR file has %s grass images but "
                "can only support %s at most" % (n_grass, LGR_GRASS_MAX)])
        if(n_pic > LGR_PIC_MAX):
            message.append([
                ERR_TOO_MANY_PICTURES,
                n_pic,
                "The LGR file has %s picture images but "
                "can only support %s at most" % (n_pic, LGR_PIC_MAX)])
        if(n_tex < LGR_TEX_MIN):
            message.append([
                ERR_NOT_ENOUGH_TEXTURES,
                n_tex,
                "The LGR file has %s texture images but "
                "needs at least %s" % (n_tex, LGR_TEX_MIN)])
        elif(n_tex > LGR_TEX_MAX):
            message.append([
                ERR_TOO_MANY_TEXTURES,
                n_tex,
                "The LGR file has %s texture image(s) but "
                "can only support %s at most" % (n_tex, LGR_TEX_MAX)])
        if(n_mask > LGR_MASK_MAX):
            message.append([
                ERR_TOO_MANY_MASKS,
                n_mask,
                "The LGR file has %s mask images but "
                "can only support %s at most" % (n_mask, LGR_MASK_MAX)])
        appleFinished = False
        for i in range(9):
            if(apples[i]):
                if(appleFinished):
                    message.append([
                        WARN_UNUSED_QFOOD,
                        None,
                        "Warning: qfood%s will not appear in "
                        "the game as qfood%s is missing" % (i+1, i)])
            else:
                appleFinished = True

        return message
    elif(type(lgro).__name__ == "LGR_Image"):
        is_mask = False
        len_name = len(lgro.name)
        if(len_name > 8):
            message.append([
                ERR_NAME_TOO_LONG,
                lgro,
                ("The name of %s is too long "
                 "(maximum 8 characters)") % lgro.name])
        if(len_name == 0):
            message.append([
                ERR_NAME_MISSING,
                lgro,
                "An LGR_Image has no name! %s" % lgro])
        try:
            if(len(bytes(lgro.padding)) != 7):
                message.append([
                    ERR_PADDING_INVALID,
                    lgro,
                    "%s's padding is invalid (must be an "
                    "array of 7 ints in range (0-255))" % lgro.name])
        except(ValueError, TypeError):
            message.append([
                ERR_PADDING_INVALID,
                lgro,
                "%s's padding is invalid (must be an "
                "array of 7 ints in range (0-255))" % lgro.name])
        namelower = lgro.name.lower()
        if(lgro.is_object()):
            if(lgro.img.height != LGR_OBJ_HEIGHT):
                message.append([
                    WARN_OBJ_HEIGHT_INVALID,
                    lgro,
                    "Warning: %s should have a height of %s, "
                    "but the height is %s. If the height is "
                    "smaller, the program might crash unexpectedly. "
                    "If the height is larger, the additional columns "
                    "will be ignored." % (
                        lgro.name,
                        LGR_OBJ_HEIGHT,
                        lgro.img.height)])
            if(lgro.img.width % LGR_OBJ_WIDTH != 0):
                message.append([
                    ERR_OBJ_WIDTH_INVALID,
                    lgro,
                    "As %s is an object, the width (%s) must be a "
                    "multiple of %s" % (
                        lgro.name,
                        lgro.img.width,
                        LGR_OBJ_WIDTH)])
            if(lgro.img.width > LGR_OBJ_WIDTH*LGR_OBJ_FRAMES_MAX):
                message.append([
                    ERR_OBJ_TOO_WIDE,
                    lgro,
                    "%s must have a width at most %s, "
                    "but the width is %s" % (
                        lgro.name,
                        lgro.img.width,
                        LGR_OBJ_WIDTH * LGR_OBJ_FRAMES_MAX)])
        elif(lgro.is_in_pictures_lst()):
                if(lgro.is_qup_qdown()):
                    if(lgro.img.height < LGR_WARNING_GRASS_HEIGHT_MIN):
                        message.append([
                            WARN_GRASS_HEIGHT_TOO_SMALL,
                            lgro,
                            "Warning: %s should have a minimum "
                            "height of %s, but the height is %s" % (
                                lgro.name,
                                LGR_WARNING_GRASS_HEIGHT_MIN,
                                lgro.img.height)])
                else:
                    if(100 > lgro.image_type < 102):
                        message.append([
                            ERR_IMAGE_TYPE_INVALID,
                            lgro,
                            "%s's image_type is invalid" % lgro.name])
                    else:
                        if(lgro.image_type == LGR_Image.PICTURE):
                            if(lgro.img.width > LGR_PIC_WIDTH_MAX):
                                message.append([
                                    ERR_PIC_TOO_WIDE,
                                    lgro,
                                    "%s is too wide (%s) - the maximum is %s" %
                                    (lgro.name,
                                     lgro.img.width,
                                     LGR_PIC_WIDTH_MAX)])
                            if(lgro.img.width * lgro.img.height >
                                    LGR_PIC_SIZE_MAX):
                                message.append([
                                    ERR_PIC_TOO_MANY_PIXELS,
                                    lgro,
                                    "%s has too many pixels (%s) - the "
                                    "maximum is %s. Depending on the image, "
                                    "the game might crash" % (
                                        lgro.name,
                                        lgro.img.width * lgro.img.height,
                                        LGR_PIC_SIZE_MAX)])
                        elif(lgro.image_type == LGR_Image.MASK):
                            is_mask = True
                    if(not(1 <= lgro.default_distance <= 999) and
                            not(is_mask)):
                        message.append([
                            ERR_DISTANCE_INVALID,
                            lgro,
                            "%s's distance (%s) is invalid "
                            "(must be integer between 1-999)" % (
                                lgro.name,
                                lgro.default_distance)])
                    if(not(0 <= lgro.default_clipping <= 2) and not(is_mask)):
                        message.append([
                            ERR_CLIPPING_INVALID,
                            lgro,
                            "%s's clipping (%s) is invalid" % (
                                lgro.name,
                                lgro.default_clipping)])
                    if(not(11 <= lgro.transparency <= 15)):
                        message.append([
                            ERR_TRANSPARENCY_INVALID,
                            lgro,
                            "%s's transparency (%s) is invalid" % (
                                lgro.name,
                                lgro.transparency)])

        if(lgro.img):
            with io.BytesIO() as f:
                lgro.save_PCX(f)
                size = f.tell()
                if(size > LGR_PCX_FILESIZE_MAX):
                    message.append([
                        ERR_FILE_TOO_LARGE,
                        lgro,
                        "The file produced by %s is %s bytes in size, "
                        "over the limit of %s" % (
                            lgro.name,
                            size,
                            LGR_PCX_FILESIZE_MAX)])
            if((namelower in LGR_LIMITED_SIZE_FILES) and
                    ((lgro.img.width > 255)or(lgro.img.height > 255))):
                message.append([
                    ERR_SMALL_IMAGE_TOO_LARGE,
                    lgro,
                    "%s must have dimensions at most 255x255, "
                    "but the dimensions are %sx%s" % (
                        lgro.name,
                        lgro.img.width,
                        lgro.img.height)])
            if(lgro.is_valid_palette_image()):
                if(palette and not is_mask and
                        (lgro.get_palette() != palette)):
                    message.append([
                        WARN_PALETTE_MISMATCH,
                        lgro,
                        ("Warning: %s's palette does not "
                         "match the LGR's palette!") % lgro.name])
            else:
                message.append([
                    ERR_IMAGE_INVALID_PALETTE,
                    lgro,
                    "%s has an invalid palette" % lgro.name])
            if(namelower == "qcolors"):
                if(lgro.img.width != LGR_WARNING_QCOLORS_WIDTH or
                        lgro.img.height != LGR_WARNING_QCOLORS_HEIGHT):
                    message.append([
                        WARN_QCOLORS_WRONG_SIZE,
                        lgro,
                        "Warning: qcolors usually has dimensions of %sx%s, "
                        "but the dimensions are %sx%s" % (
                            LGR_WARNING_QCOLORS_WIDTH,
                            LGR_WARNING_QCOLORS_HEIGHT,
                            lgro.img.width,
                            lgro.img.height)])
            if(namelower == "q1bike" or namelower == "q2bike"):
                if(lgro.img.width < LGR_RECOMMEND_BIKE_WIDTH or
                        lgro.img.height < LGR_RECOMMEND_BIKE_HEIGHT):
                    message.append([
                        WARN_QBIKE_TOO_SMALL,
                        lgro,
                        "Warning: %s needs dimensions of at least %sx%s to be "
                        "rendered properly, but the dimensions are %sx%s" % (
                            lgro.name,
                            LGR_RECOMMEND_BIKE_WIDTH,
                            LGR_RECOMMEND_BIKE_HEIGHT,
                            lgro.img.width,
                            lgro.img.height)])
        else:
            message.append([
                ERR_IMG_MISSING,
                lgro,
                "%s has no image!" % lgro.name])
    else:
        raise ValueError("only LGR and LGR_Image objects can be "
                         "evaluated with this function")
    return message
